@@ -1,5 +1,6 @@
-// Sound recorder 0.05 Aug 01 1998, GPL 2 (see COPYRIGHTS)
+// Sound recorder 0.06 (Build on Aug 06 2000), GPL 2 (see COPYRIGHTS)
 // 1997, 1998  B. Warmerdam
+// $Id: pcm.cc,v 1.1.1.1 1999/02/20 14:19:02 bartw Exp $
 
 #include "pcm.h"
 
@@ -16,13 +17,19 @@ TPcm::~TPcm()
 }
 
 /* Create a new file with specified arguments */
-const bool TPcm::create(const char * filename, unsigned, unsigned, unsigned, unsigned)
+const bool TPcm::create(const char * filename, wavfmt_t, chnl_t, bps_t, smpl_t)
 {
 	if(fh == 0){
-		if((fh = ::fopen(filename, "w+b")) == 0){
-			perror("Wave create error");
-		} else {
+		if(filename[0] == '-' && filename[1] == '\0') {
+			fprintf(stderr, "Writing to stdout.\n");
+			fh = stdout;
 			mode = O_WRONLY;
+		} else {
+			if((fh = ::fopen(filename, "w+b")) == 0){
+				perror("PCM create error");
+			} else {
+				mode = O_WRONLY;
+			}
 		}
 	}
 	return fh != 0;
@@ -32,17 +39,23 @@ const bool TPcm::create(const char * filename, unsigned, unsigned, unsigned, uns
 const bool TPcm::open(const char * filename)
 {
 	if(fh == 0){
-		if((fh = ::fopen(filename, "rb")) == 0){
-			perror("Pcm read error");
-		} else {
+		if(filename[0] == '-' && filename[1] == '\0') {
+			fprintf(stderr, "Reading from stdin.\n");
+			fh = stdin;
 			mode = O_RDONLY;
+		} else {
+			if((fh = ::fopen(filename, "rb")) == 0){
+				perror("Pcm read error");
+			} else {
+				mode = O_RDONLY;
+			}
 		}
 	}
 	return fh != 0;
 }
 
 /* Write data to a pcm-file */
-const int TPcm::write(char * buffer, unsigned long len)
+const int TPcm::write(char * buffer, int len)
 {
 	int size = 0;
 
@@ -53,7 +66,7 @@ const int TPcm::write(char * buffer, unsigned long len)
 }
 
 /* Read data from a pcm file */
-const int TPcm::read(char * buffer, unsigned long len)
+const int TPcm::read(char * buffer, int len)
 {
 	int size = 0;
 
