@@ -1,5 +1,6 @@
-// Sound recorder 0.05 Aug 01 1998, GPL 2 (see COPYRIGHTS)
+// Sound recorder 0.06 (Build on Aug 06 2000), GPL 2 (see COPYRIGHTS)
 // 1997, 1998  B. Warmerdam
+// $Id: record.cc,v 1.1.1.1 1999/02/20 14:19:02 bartw Exp $
 
 #include "record.h"
 
@@ -8,33 +9,34 @@ int stop;				// Arggg... Shitty global.
 /* Print the available commandline options */
 void printOptions(bool useCDrom)
 {
-	cout << "Options:"
-		"\n\t-c\tNumber of channels"
+	cerr << "Options:"
+		"\n\t-c\tNumber of channels [1/2]"
 		"\n\t-s\tSamplerate of the recording"
 		"\n\t-b\tBits per sample [8/16]"
 		"\n\t-k\tKeep going if destination file already exists (overwrite)"
 		"\n\t-P\tUse a higher priority for recording thread"
-		"\n\t-f\tOutput format [wav/pcm]" << ends;
+		"\n\t-A\tAudio device (default /dev/dsp)"
+		"\n\t-e\tExecute this statement after recording (eg 'rm $file')"
+		"\n\t-S\tThe recording time (mm:ss)"
+		"\n\t-f\tOutput format [wav/pcm/cdr/ima3/ima4/ima5]" << ends;
 	if(useCDrom == true){
-		cout << "\n\t-t\tTrack to start recording"
+		cerr << "\n\t-t\tTrack to start recording"
 			"\n\t-p\tTrack to stop recording"
 			"\n\t-l\tTracklist to record; comma separated list (no space)"
-			"\n\t-e\tExecute this statement after recording (eg 'rm $file')"
 			"\n\t-C\tFull cdrom recording (seperate files, ??_file)"
-			"\n\t-S\tThe number of samples to record in samplerate"
-			"\n\t-O\tStarttime of a sample (MM:SS)"
+			"\n\t-O\tStarttime of a sample (mm:ss)"
 			"\n\t-q\tOutput no track information to screen"
 			"\n\t-D\tCDROM device (default /dev/cdrom)"
 			"\n\t-M\tMixer device (default /dev/mixer)"
-			"\n\t-A\tAudio device (default /dev/dsp)"
 			"\n\t-Q\tQuit on exec-error" << ends;
 	}
 
-	cout << "\n\t-h\tThis information"
+	cerr << "\n\t-h\tThis information"
 		"\n" << endl;
 		       
 }
 
+/* Test if the file exists and can be created (exists) */
 const bool fileCanBeCreated(struct recordSettings * recSettings)
 {
 	struct stat fstat;
@@ -49,23 +51,68 @@ const bool fileCanBeCreated(struct recordSettings * recSettings)
 	return status;
 }
 
-/* Read from /dev/dsp the audio and record it in a file */
+/* Read from audio device to an intermediat, rotating buffer to reduce
+   the numbers of DMA buffer overrun errors.
+
+   Patch submitted by:  Frank Heckenbach <frank@tim.gerwinski.de> */
+void * recordRead (volatile struct audio_buffers *buffers)
+{
+	time_t start_time = time(0);
+	bool countSeconds = buffers->numberOfSeconds > 0;
+	buffers->recordedBytes = 0L;
+	while((buffers->len [buffers->last] = buffers->dsp->read(buffers->buf [buffers->last], buffers->bufferSize)) > 0){
+		if(countSeconds && (unsigned long)(time(0) - start_time) > buffers->numberOfSeconds) {
+			buffers->len[buffers->last] =
+				(buffers->len[buffers->last] * buffers->numberOfSeconds) / (time(0) - start_time);
+		}
+
+		buffers->recordedBytes += buffers->len [buffers->last];
+		if ((buffers->last = (buffers->last + 1) % NUM_BUFFERS) == buffers->first) {
+			fputs("Recording overrun\n", stderr);
+			while (buffers->last == buffers->first)
+		      		sched_yield();
+		}
+		write(buffers->pipe[1], "\1", 1);
+
+		if(countSeconds && buffers->numberOfSeconds < (unsigned long)(time(0) - start_time)) stop = 1;
+		if(stop) break;
+	}
+	write(buffers->pipe[1], "\0", 1);
+	return NULL;
+}
+
+/* Write audio to output using the intermediate buffer
+
+   Patch submitted by:  Frank Heckenbach <frank@tim.gerwinski.de> */
+void * recordWrite (volatile struct audio_buffers *buffers)
+{
+	char c;
+	while (1) {
+		read(buffers->pipe[0], &c, 1);
+		if (!c)
+			break;
+		buffers->oFile->write(buffers->buf [buffers->first], buffers->len [buffers->first]);
+		buffers->first = (buffers->first + 1) % NUM_BUFFERS;
+		if (buffers->first != buffers->last)
+			printf("Writing behind by %i buffers\n", (buffers->last - buffers->first + NUM_BUFFERS) % NUM_BUFFERS);
+	}
+	return NULL;
+}
+
+/* Read from /dev/dsp the audio and record it in a file
+
+   Partly patched function.
+   Patch submitted by:  Frank Heckenbach <frank@tim.gerwinski.de> */
 void recordIt(struct recordSettings * recSettings)
 {
-	char * audio_buffer;
-	unsigned long len;
-	unsigned long bufferSize;
-	unsigned long recordedSamples = 0L;
-	unsigned long numberOfSamples = recSettings->getNumberOfSamples();
-
-	TFileFormat * oFile = 0;
+	struct audio_buffers buffers;
+	buffers.oFile = 0;
 
 	if(recSettings->getHighPrioOnAudioThr() == true){
-		int policy;
+		int policy = SCHED_FIFO;
 		pthread_attr_t attr;
 		struct sched_param sparam;
 		
-		pthread_getschedparam(pthread_self(), & policy, & sparam);
 		sparam.sched_priority = sched_get_priority_max(policy);
 		pthread_attr_init(&attr);
 		pthread_attr_setschedparam(&attr, &sparam);
@@ -73,43 +120,62 @@ void recordIt(struct recordSettings * recSettings)
 
 	DspSetting dspSetting(recSettings->getSoundDevice(), recSettings->getSampleRate(), O_RDONLY, 
 				recSettings->getRecordFormat(), recSettings->getChannels());
-	Dsp dsp(dspSetting);
+
+	buffers.numberOfSeconds = recSettings->getNumberOfSeconds();
+	buffers.dsp = new Dsp(dspSetting);
 
 	switch(recSettings->getFileFormat()){
 		case WAV:
-			oFile = new TWave;
+		case IMA2:
+		case IMA3:
+		case IMA4:
+		case IMA5:
+		case MSADPCM:
+			buffers.oFile = new TWave;
 			break;
 		case PCM:
-			oFile = new TPcm;
+			buffers.oFile = new TPcm;
+			break;
+		case CDR:
+			buffers.oFile = new TCdr;
 	}
 
-	if(dsp.init()){
+	if(buffers.dsp->init()){
 		if(fileCanBeCreated(recSettings) == true){
-			bufferSize = dsp.getOptimalBufferSize();
-			cout << "Creating file: " << recSettings->getIoFile() << "\n" << endl;
+			const int bps = recSettings->getBitsPerSample() / 8;
+			buffers.bufferSize = buffers.dsp->getOptimalBufferSize();
+			cerr << "Creating file: " << recSettings->getIoFile() << "\n" << endl;
 
-			audio_buffer = new char[bufferSize];
-			if(oFile->create(recSettings->getIoFile(), 1, recSettings->getChannels(),
-					 recSettings->getBitsPerSample(), recSettings->getSampleRate()) == false){
+			for (int i = 0; i < NUM_BUFFERS; i++)
+				buffers.buf [i] = (char *) malloc(buffers.bufferSize);
+			buffers.first = buffers.last = 0;
+
+			if(buffers.oFile->create(recSettings->getIoFile(), 
+					recSettings->getFileFormatId(), 
+					recSettings->getChannels(),
+					recSettings->getOutputBitsPerSample(), 
+					recSettings->getSampleRate()) == false){
 				perror("Outputfile");
-			} else {
-				bool countSamples = numberOfSamples > 0;
-				while((len = dsp.read(audio_buffer, bufferSize)) > 0){
-					if(countSamples == true && len > numberOfSamples)
-						len = numberOfSamples;
-
-					oFile->write(audio_buffer, len);
-					numberOfSamples -= len;
-					recordedSamples += len;
-
-					if(countSamples == true && numberOfSamples <= 0) stop = 1;
-					if(stop) break;
+			} else { 
+				if(pipe(buffers.pipe)) {
+					perror("Pipe");
+				} else {
+					pthread_t pa, pb;
+					pthread_create(&pa,0,(void *(*)(void *)) recordRead,  (void *) &buffers);
+					pthread_create(&pb,0,(void *(*)(void *)) recordWrite, (void *) &buffers);
+					pthread_join(pa, (void **) NULL);
+					pthread_join(pb, (void **) NULL);
+					close (buffers.pipe[0]);
+					close (buffers.pipe[1]);
 				}
 			}
-			delete [] audio_buffer;
 
-			cout << "\nEnd of recording: " << recSettings->getIoFile() <<
-				" with " << recordedSamples << " samples" << endl;
+			for (int i = 0; i < NUM_BUFFERS; i++)
+				free(buffers.buf[i]);
+
+			cerr << "\nEnd of recording: " << recSettings->getIoFile() <<
+				" with " << buffers.recordedBytes / bps <<
+				" samples" << endl;
 		} else {
 			cerr << "File can't be created (use -k to override "
 				"overwrite protection)." << endl;
@@ -120,8 +186,9 @@ void recordIt(struct recordSettings * recSettings)
 		cerr << "Error during initialisation of soundcard." << endl;
 	}
 
-	oFile->close();
-	delete oFile;
+	buffers.oFile->close();
+	delete buffers.oFile;
+	delete buffers.dsp;
 }
 
 /* Show info from cdrom-recording  */
@@ -129,17 +196,16 @@ void cdromShow(struct cdRecordData * cdRecData)
 {
 	CDplayer::CDtrack track;
 	int time = 0;
-	int savedCurrentTrack;
-	int trk  = 0;
-	int indx = 0;
+	cdtrck_t savedCurrentTrack;
+	cdtrck_t trk  = 0;
+	cdindx_t indx = 0;
 	bool quiet = cdRecData->recSettings->getQuiet();
 
 	if(cdRecData->recSettings->getHighPrioOnAudioThr() == true){
-		int policy;
+		int policy = SCHED_FIFO;
 		pthread_attr_t attr;
 		struct sched_param sparam;
 		
-		pthread_getschedparam(pthread_self(), & policy, & sparam);
 		sparam.sched_priority = sched_get_priority_min(policy);
 		pthread_attr_init(&attr);
 		pthread_attr_setschedparam(&attr, &sparam);
@@ -159,7 +225,7 @@ void cdromShow(struct cdRecordData * cdRecData)
 			time--;
 		}
 		if(quiet == false){
-			cout << "Number " << trk << ":" << 
+			cerr << "Number " << trk << ":" << 
 				setw(2) << setfill('0') << indx << 
 				" " << time / 60 << ":" << 
 				setw(2) << setfill('0') <<time % 60 <<
@@ -176,15 +242,15 @@ void cdromShow(struct cdRecordData * cdRecData)
 /* Show info from line-recording, stops on signal */
 void lineShow(struct cdRecordData * cdRecData)
 {
-	int time = 1;
+	time_t start_time = time(0);
+	int current_time = 1;
 	bool quiet = cdRecData->recSettings->getQuiet();
 
 	if(cdRecData->recSettings->getHighPrioOnAudioThr() == true){
-		int policy;
+		int policy = SCHED_FIFO;
 		pthread_attr_t attr;
 		struct sched_param sparam;
 		
-		pthread_getschedparam(pthread_self(), & policy, & sparam);
 		sparam.sched_priority = sched_get_priority_min(policy);
 		pthread_attr_init(&attr);
 		pthread_attr_setschedparam(&attr, &sparam);
@@ -192,11 +258,18 @@ void lineShow(struct cdRecordData * cdRecData)
 	sleep(1);
 	do {
 		if(quiet == false){
-			cout << "Time " << setw(2) << setfill('0') << 
-				time / 60 << ":" << time % 60 << "\r" << ends;
+			cerr << "Time " <<
+				setw(2) << setfill('0') <<
+				current_time / 60 << ":" << 
+				setw(2) << setfill('0') <<
+				current_time % 60 << "\r" << flush;
 		}
 		sleep(1);
-		time = time + 1;
+		if(current_time % 10 == 0) {
+			current_time = time(0) - start_time;
+		} else {
+			current_time++;
+		}
 	} while(!stop);
 }
 
@@ -219,7 +292,7 @@ void connect_signal(int the_signal)
 }
 
 /* Get the next track from the track-record list */
-int getNextTrack(unsigned & trackIdx, struct recordSettings & recSettings)
+int getNextTrack(cdtrck_t & trackIdx, struct recordSettings & recSettings)
 {
 	if(trackIdx >= 1 && trackIdx <= 99){
 		for(; trackIdx <= 99; trackIdx++){
@@ -260,7 +333,8 @@ void checkMixerSettingsForCd(struct recordSettings * recSettings)
 void cdrom_record(struct recordSettings * recSettings)
 {
 	void * retval;
-	unsigned current_track, trackIdx;
+	cdtrck_t current_track;
+	cdindx_t trackIdx;
 	pthread_t pa, pb;
 	char file[256];
 	const char * iofile = recSettings->getIoFile();
@@ -299,14 +373,14 @@ void cdrom_record(struct recordSettings * recSettings)
 					pthread_join(pb,(void **)&retval);
 
 					if(stop == 1 && recSettings->getExecStmt() != 0){
-						char * cmd = new char[50 + strlen(recSettings->getIoFile()) + 
-								strlen(recSettings->getExecStmt())];
-						sprintf(cmd, "declare -x file=%s;%s", recSettings->getIoFile(),
-							recSettings->getExecStmt());
-
 						int pid, status;
 						if(!(pid = fork())){
-							execl("/bin/sh", "sh", "-c", cmd, 0);
+							/* Patch setenv by
+							 * Michael Fowler
+							 * <michael@shoebox.net>
+							 * */
+							setenv("file", recSettings->getIoFile(), 1);
+							execl("/bin/sh", "sh", "-c", recSettings->getExecStmt(), 0);
 							exit(0);
 						}
 
@@ -316,8 +390,6 @@ void cdrom_record(struct recordSettings * recSettings)
                                                         cerr << "Child exits with " << status << endl;
                                                         stop = 2;
                                                 }
- 
-						delete [] cmd;
 					}
 					if(stop == 1) stop = 0;
 				} else {
@@ -347,7 +419,7 @@ void line_record(struct recordSettings * recSettings)
 		connect_signal(SIGINT);		// for now
 		connect_signal(SIGTERM);
 
-		cout << "Record from dsp (no cdrom support).\nTo end the recording press CTRL-C" << endl;
+		cerr << "Record from dsp (no cdrom support).\nTo end the recording press CTRL-C" << endl;
 
 		cdRecData.cdrom = 0;
 		cdRecData.recSettings = recSettings;
@@ -356,6 +428,25 @@ void line_record(struct recordSettings * recSettings)
 		pthread_create(&pb,0,(void *(*)(void *)) lineShow, &cdRecData);
 		pthread_join(pa,(void **)&retval);
 		pthread_join(pb,(void **)&retval);
+
+		if(recSettings->getExecStmt() != 0){
+			int pid, status;
+			if(!(pid = fork())){
+				/* Patch setenv by Michael Fowler
+				 * <michael@shoebox.net> */
+				setenv("file", recSettings->getIoFile(), 1);
+				execl("/bin/sh", "sh", "-c", recSettings->getExecStmt(), 0);
+				exit(0);
+			}
+
+			waitpid(pid, &status, 0);
+
+			if(status != 0 && recSettings->getQuitOnError() == true){
+				cerr << "Child exits with " << status << endl;
+				stop = 2;
+			}
+		}
+
 	}
 }
 
@@ -379,15 +470,24 @@ void readSettings(RCfile & rc, recordSettings  & recSettings)
 
 		if(rc.getEntry("samplerate", buffer, sizeof(buffer)) == true){
 
-			unsigned sampleRate;
+			u_int32_t sampleRate;
 			sscanf(buffer, "%ul", &sampleRate);
 
 			recSettings.setSampleRate(sampleRate);
 		}
 
 		if(rc.getEntry("fileformat", buffer, sizeof(buffer)) == true){
-			if(!strcmp(buffer, "wave")) recSettings.setFileFormat(WAV);
-			if(!strcmp(buffer, "pcm")) recSettings.setFileFormat(PCM);
+			OUTPUT_FORMATS fmt = NONE;
+			if(!strcmp(buffer, "wave")) fmt = WAV;
+			if(!strcmp(buffer, "pcm")) fmt = PCM;
+			if(!strcmp(buffer, "cdr")) fmt = CDR;
+
+			if(fmt == NONE){
+				fmt = WAV;
+				cerr << "Format " << buffer << " unknown." <<
+					"Using default format WAV." << endl;
+			}
+			recSettings.setFileFormat(fmt);
 		}
 
 		if(rc.getEntry("format", buffer, sizeof(buffer)) == true){
@@ -400,8 +500,10 @@ void readSettings(RCfile & rc, recordSettings  & recSettings)
 				};
 
 			for(int i = 0; i < 9; i++){
-				if(!strcmp(buffer, r[i].name))
+				if(!strcmp(buffer, r[i].name)){
 					recSettings.setRecordFormat(r[i].format);
+					break;
+				}
 			}
 		}
 
@@ -455,7 +557,7 @@ void defaultSettings(recordSettings & recSettings)
 /* Create a list with the tracknumbers from -tlst argument */
 void expandList(struct recordSettings & recSettings, char * track_list)
 {
-	unsigned i = 0, from, to = 1, dst = 1;
+	u_int32_t i = 0, from, to = 1, dst = 1;
 	while(i < strlen(track_list)){
 		from = 0;
 		if(track_list[i] < '0' || track_list[i] > '9'){
@@ -468,7 +570,7 @@ void expandList(struct recordSettings & recSettings, char * track_list)
 		sscanf(track_list + i,"%d", & to);
 		while(track_list[i] >= '0' && track_list[i] <= '9') i++;
 		if(from){
-			for(unsigned j = from + 1; j <= to; j++)
+			for(u_int32_t j = from + 1; j <= to; j++)
 				recSettings.track_list[dst++] = j;
 		} else {
 			recSettings.track_list[dst++] = to;
@@ -478,11 +580,11 @@ void expandList(struct recordSettings & recSettings, char * track_list)
 
 const bool threadScheduleAvailable()
 {
-#ifdef _POSIX_THREAD_PRIORITY_SCHEDULING
-	return true;
-#else
-	return false;
-#endif
+	printf("Using POSIX Priority Threading.\n");
+	if(sysconf(_SC_THREAD_PRIORITY_SCHEDULING) == 1)
+		return true;
+	else
+		return false;
 }
 
 /* Extract extra parameters for recordsettings */
@@ -491,9 +593,10 @@ bool parseArgs(int argc, char ** argv, recordSettings & recSettings, bool useCDr
 	bool status = true;
 	long lng;
 	int opt, i, min, sec, tstt = 0, tstp = 0;
+	u_int16_t forceBpsTo = 0;
 	defaultSettings(recSettings);
 
-	while((opt = getopt(argc, argv, useCDrom ? "c:s:f:t:p:l:e:S:O:D:M:A:b:ChkPqQ" : "c:s:f:b:hPq")) != -1){
+	while((opt = getopt(argc, argv, useCDrom ? "c:s:f:t:p:l:e:S:O:D:M:A:b:ChkPqQ" : "c:s:f:e:S:b:A:hPq")) != -1){
 		switch(opt){
 			case '?':
 			case 'h':
@@ -508,8 +611,56 @@ bool parseArgs(int argc, char ** argv, recordSettings & recSettings, bool useCDr
 				status &= recSettings.setSampleRate(lng);
 				break;
 			case 'f':
-				if(!strcasecmp("wav", optarg)) status &= recSettings.setFileFormat(WAV);
-				if(!strcasecmp("pcm", optarg)) status &= recSettings.setFileFormat(PCM);
+				if(!strcasecmp("wav", optarg)){
+					status &= recSettings.setFileFormat(WAV);
+					recSettings.setFileFormatId(0x0001);
+				}
+
+				if(!strcasecmp("pcm", optarg)){
+					status &= recSettings.setFileFormat(PCM);
+					recSettings.setFileFormatId(0x0001);
+				}
+				
+				if(!strcasecmp("cdr", optarg)){
+					status &= recSettings.setFileFormat(CDR);
+					recSettings.setFileFormatId(0x0001);
+				}
+				
+				if(!strcasecmp("ima2", optarg)){ 
+					status &= recSettings.setFileFormat(IMA2);
+					forceBpsTo = 16;
+					recSettings.setFileFormatId(0x0011);
+					recSettings.setOutputBitsPerSample(2);
+				}
+				
+				if(!strcasecmp("ima3", optarg)){
+					status &= recSettings.setFileFormat(IMA3);
+					forceBpsTo = 16;
+					recSettings.setFileFormatId(0x0011);
+					recSettings.setOutputBitsPerSample(3);
+				}
+				
+				if(!strcasecmp("ima4", optarg)){
+					status &= recSettings.setFileFormat(IMA4);
+					forceBpsTo = 16;
+					recSettings.setFileFormatId(0x0011);
+					recSettings.setOutputBitsPerSample(4);
+				}
+				
+				if(!strcasecmp("ima5", optarg)){
+					status &= recSettings.setFileFormat(IMA5);
+					forceBpsTo = 16;
+					recSettings.setFileFormatId(0x0011);
+					recSettings.setOutputBitsPerSample(5);
+				}
+				
+				if(!strcasecmp("msadpcm", optarg)){
+					status &= recSettings.setFileFormat(MSADPCM);
+					forceBpsTo = 16;
+					recSettings.setFileFormatId(0x0002);
+					recSettings.setOutputBitsPerSample(4);
+				}
+				
 				break;
 			case 'b':
 				sscanf(optarg, "%d", &i);
@@ -541,8 +692,14 @@ bool parseArgs(int argc, char ** argv, recordSettings & recSettings, bool useCDr
 					recSettings.track_list[i] = i;
 				break;
 			case 'S':
-				sscanf(optarg,"%lu", &lng);
-				status &= recSettings.setNumberOfSamples(lng);
+				if(sscanf(optarg, "%d:%d", &min, &sec) == 2) {
+					cerr << "Setting recording limit to "
+						<< (60 * min + sec) << " seconds\n";
+					status &= recSettings.setNumberOfSeconds(60 * min + sec);
+				} else {
+					cerr << "Error in timedefinition -S\n";
+					status = -1;
+				}
 				break;
 			case 'O':
 				sscanf(optarg, "%d:%d", &min, &sec);
@@ -574,7 +731,16 @@ bool parseArgs(int argc, char ** argv, recordSettings & recSettings, bool useCDr
 
 	status &= recSettings.setIoFile(argv[optind]);
 
-	if(recSettings.getHighPrioOnAudioThr() == true && threadScheduleAvailable() == true){
+	if(forceBpsTo){
+		if(recSettings.getBitsPerSample() != forceBpsTo){
+			cerr << "Output format forces change of inputformat."
+			     << endl << "Using " << forceBpsTo << " bit p/s"
+			     << endl;
+			recSettings.setBitsPerSample(forceBpsTo);
+		}
+	}
+
+	if(recSettings.getHighPrioOnAudioThr() == true && threadScheduleAvailable() == false){
 		cerr << "Priority scheduling not supported on this system." << endl;
 		recSettings.setHighPrioOnAudioThr(false);
 	}
@@ -589,20 +755,26 @@ bool parseArgs(int argc, char ** argv, recordSettings & recSettings, bool useCDr
 /* Commandline coordinated use of binary */
 int main(int argc, char **argv)
 {
-	cout << "Sound Recorder version " VERSION "\n"
-		"Copyright (C) 1997, 1998 by B. Warmerdam under GPL.\n"
+	cerr << "Sound Recorder version " VERSION "\n"
+		"Copyright (C) 1997-2000 by B. Warmerdam under GPL.\n"
 		"This program is free software and comes with ABSOLUTELY NO WARRANTY.\n" << endl;
 
-	struct recordSettings recSettings;
-	bool useCDrom = (strlen(argv[0]) >= 16 && ! strcmp(argv[0] + 
-		strlen(argv[0]) - 16, "cdsound-recorder"));
+	int len = strlen(argv[0]);
+	bool useCDrom = (len >= 16 && ! strcmp(argv[0] + len - 16, "cdsound-recorder"));
 
-	if(parseArgs(argc, argv, recSettings, useCDrom) == true){
-		if(useCDrom == true){
-			cdrom_record(& recSettings);
-		} else {
-			line_record(& recSettings);
+	try{
+		struct recordSettings recSettings;
+		if(parseArgs(argc, argv, recSettings, useCDrom) == true){
+			if(useCDrom == true){
+				cdrom_record(& recSettings);
+			} else {
+				line_record(& recSettings);
+			}
 		}
 	}
+	catch(char * err){
+		cerr << "Error: " << err << endl;
+	}
 	return 0;
 }
+
